@@ -279,7 +279,7 @@ correct_localedir_prefix (gchar **path)
 #endif
 
 
-static GtkIMModule *
+G_GNUC_UNUSED static GtkIMModule *
 add_builtin_module (const gchar             *module_name,
 		    const GtkIMContextInfo **contexts,
 		    int                      n_contexts)
@@ -682,20 +682,16 @@ _gtk_im_module_get_default_context_id (GdkWindow *client_window)
 
   /* Check if the certain immodule is set in XSETTINGS.
    */
-  if (client_window != NULL && GDK_IS_DRAWABLE (client_window))
+  if (GDK_IS_DRAWABLE (client_window))
     {
       screen = gdk_drawable_get_screen (GDK_DRAWABLE (client_window));
-      if (screen)
-        settings = gtk_settings_get_for_screen (screen);
-      else
-        settings = gtk_settings_get_default ();
-
+      settings = gtk_settings_get_for_screen (screen);
       g_object_get (G_OBJECT (settings), "gtk-im-module", &tmp, NULL);
       if (tmp)
         {
           if (strcmp (tmp, SIMPLE_ID) == 0)
             context_id = SIMPLE_ID;
-          else 
+          else
             {
               GtkIMModule *module;
               module = g_hash_table_lookup (contexts_hash, tmp);
@@ -704,7 +700,7 @@ _gtk_im_module_get_default_context_id (GdkWindow *client_window)
             }
           g_free (tmp);
 
-       	  if (context_id) 
+       	  if (context_id)
             return context_id;
         }
     }
